﻿/*
Class:
    df.WebMenuItem
Extends:
    df.WebBaseUIObject

This is the client-side representation of the WebMenuBar class that wraps the WebMenuItems. It 
generates the HTML for the ul element that wraps the li elements of the WebMenuItems. The CSS 
(class: WebMenuBar) Determines how the menu is shown and how the items will look depending on their 
place in the menu structure.
    
Revision:
    2011/10/04  (HW, DAW) 
        Initial version.
*/

df.WebMenuBar = function WebMenuBar(oDef, oParent){
    df.WebMenuBar.base.constructor.call(this, oDef, oParent);
    
    // @privates
    this._eControl = null;
    
    this._bWrapDiv = true;
    this._bRenderChildren = true;
    this._sControlClass = "WebMenuBar";
};
/*
This class represents a menubar inside a command bar. It belongs to the server-side cWebMenuBar 
class and implements its functionality on the client. The CSS (class: WebMenuBar) defines the looks 
of this control and the WebMenuItem's that it wraps.
*/
df.defineClass("df.WebMenuBar", "df.WebBaseUIObject",{

//  - - - Rendering - - -

/*
The WebMenuBar consists of a single ul element that is wrapped by the div generated by its 
base class.

@param  aHtml   String builder array to add the HTML to.

@private
*/
openHtml : function(aHtml){
    df.WebMenuBar.base.openHtml.call(this, aHtml);
     
    aHtml.push('<ul>');

},

/*
Properly close the opened HTML elements.

@param  aHtml   String builder array to add the HTML to.

@private
*/
closeHtml : function(aHtml){
    aHtml.push('</ul>');
    aHtml.push('<div style="clear: both"></div>');
     
    df.WebMenuBar.base.closeHtml.call(this, aHtml);
},

/*
We need to augment the renderChildren method because the children need to be added to the ul 
element.

@private
*/
renderChildren : function(eContainer){
    //  We need to get a reference to the ul element
    this._eControl = df.dom.query(this._eElem, "ul");

    df.WebMenuBar.base.renderChildren.call(this, this._eControl);
}

});